@@ -7,12 +7,11 @@ package DateTime::Language::Czech;
 
 use strict;
 
-use vars qw(@ISA @DayNames @DayAbbreviations @MonthNames @MonthAbbreviations @MonthNames2 @AMPM $VERSION);
-@ISA = qw(DateTime::LanguageDateTime::Format::Generic);
-
+use DateTime::Language;
+use vars qw(@ISA @DayNames @DayAbbreviations @MonthNames @MonthAbbreviations @MonthNames2 @AMPM);@ISA = qw(DateTime::Language);
 
 @MonthNames = qw(leden nor bezen duben kvten erven ervenec srpen z
-	      jen listopad prosinec);
+                 jen listopad prosinec);
 @MonthAbbreviations = qw(led nor be dub kv vn ec srp z j lis pro);
 @MonthNames2 = @MonthNames;
 for (@MonthNames2)
